/*
 *   Licensed under the Apache License, Version 2.0 (the "License");
 *   you may not use this file except in compliance with the License.
 *   You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 */

#include <Python.h>
#include "macros.h"

void installType(PyTypeObject **type, PyType_Def *def, PyObject *module,
                 char *name, int isExtension);

namespace java {
    namespace io {

        DECLARE_TYPE(Writer);
        DECLARE_TYPE(StringWriter);
        DECLARE_TYPE(PrintWriter);

        namespace reflect {
            void __install__(PyObject *module);
        }

        void __install__(PyObject *m)
        {
            INSTALL_TYPE(Writer, m);
            INSTALL_TYPE(StringWriter, m);
            INSTALL_TYPE(PrintWriter, m);
        }
    }
}
