/*
 *   Licensed under the Apache License, Version 2.0 (the "License");
 *   you may not use this file except in compliance with the License.
 *   You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 */

#include <jni.h>
#include "JCCEnv.h"

#include "java/lang/Object.h"
#include "java/lang/Class.h"
#include "java/lang/String.h"

namespace java {
    namespace lang {
        enum {
            mid__init_,
            mid_toString,
            mid_getClass,
            mid_hashCode,
            mid_equals,
            max_mid
        };

        Class *Object::class$ = NULL;
        jmethodID *Object::mids$ = NULL;

        jclass Object::initializeClass(bool getOnly)
        {
            if (getOnly)
                return (jclass) (class$ == NULL ? NULL : class$->this$);
            if (!class$)
            {
                jclass cls = env->findClass("java/lang/Object");

                mids$ = new jmethodID[max_mid];
                mids$[mid__init_] = env->getMethodID(cls, "<init>",
                                                     "()V");
                mids$[mid_toString] = env->getMethodID(cls, "toString",
                                                       "()Ljava/lang/String;");
                mids$[mid_getClass] = env->getMethodID(cls, "getClass",
                                                       "()Ljava/lang/Class;");
                mids$[mid_hashCode] = env->getMethodID(cls, "hashCode",
                                                       "()I");
                mids$[mid_equals] = env->getMethodID(cls, "equals",
                                                     "(Ljava/lang/Object;)Z");

                class$ = (Class *) new JObject(cls);
            }

            return (jclass) class$->this$;
        }

        Object::Object() : JObject(env->newObject(initializeClass, &mids$, mid__init_)) {
        }

        String Object::toString() const
        {
            return String(env->callObjectMethod(this$, mids$[mid_toString]));
        }

        Class Object::getClass() const
        {
            return Class(env->callObjectMethod(this$, mids$[mid_getClass]));
        }

        int Object::hashCode() const
        {
            return env->callIntMethod(this$, mids$[mid_hashCode]);
        }

        jboolean Object::equals(const Object& a0) const
        {
            return env->callBooleanMethod(this$, mids$[mid_equals], a0.this$);
        }
    }
}


#include "structmember.h"
#include "functions.h"
#include "macros.h"

namespace java {
    namespace lang {

        static int t_Object_init(t_Object *self,
                                 PyObject *args, PyObject *kwds);
        static PyObject *t_Object_getClass(t_Object *self);
        static PyObject *t_Object_equals(t_Object *self, PyObject *arg);

        static PyMethodDef t_Object__methods_[] = {
            DECLARE_METHOD(t_Object, getClass, METH_NOARGS),
            DECLARE_METHOD(t_Object, equals, METH_O),
            { NULL, NULL, 0, NULL }
        };

        static PyType_Slot PY_TYPE_SLOTS(Object)[] = {
            { Py_tp_methods, t_Object__methods_ },
            { Py_tp_init, (void *) t_Object_init },
            { 0, 0 }
        };

        static PyType_Def *PY_TYPE_BASES(Object)[] = {
            &PY_TYPE_DEF(JObject),
            NULL
        };

        DEFINE_TYPE(Object, t_Object, java::lang::Object);

        static int t_Object_init(t_Object *self,
                                 PyObject *args, PyObject *kwds)
        {
            switch (PyTuple_Size(args)) {
              case 0:
                INT_CALL(self->object = Object());
                break;
              default:
                PyErr_SetString(PyExc_ValueError, "invalid args");
                return -1;
            }

            return 0;
        }

        static PyObject *t_Object_getClass(t_Object *self)
        {
            Class cls((jobject) NULL);

            OBJ_CALL(cls = self->object.getClass());
            return t_Class::wrap_Object(cls);
        }

        static PyObject *t_Object_equals(t_Object *self, PyObject *arg)
        {
            Object a0((jobject) NULL);
            jboolean result;

            if (!parseArg(arg, "o", &a0))
            {
                OBJ_CALL(result = self->object.equals(a0));
                Py_RETURN_BOOL(result);
            }

            PyErr_SetArgsError((PyObject *) self, "equals", arg);
            return NULL;
        }
    }
}
